//
//  GAM.h
//  GAMKit
//
//  Created by Isaac Wankerl on 11/12/13.
//  Copyright (c) 2013 Evernote Corp. All rights reserved.
//

#import <Foundation/Foundation.h>

@protocol GAMTracker;

@interface GAM : NSObject
{
    BOOL _optOut;
    NSTimeInterval _dispatchInterval;
    BOOL _dryRun;
    NSTimer *_dispatchTimer;
    NSMutableDictionary *_trackerMap;
    NSOperationQueue *_postingQueue;
    dispatch_queue_t _enqueingQueue;
    NSMutableArray *_pendingItems;
}

/*!
 When this is true, no tracking information will be gathered; tracking calls
 will effectively become no-ops. When set to true, all tracking information that
 has not yet been submitted. The value of this flag will be persisted
 automatically by the SDK.  Developers can optionally use this flag to implement
 an opt-out setting in the app to allows users to opt out of Google Analytics
 tracking.

 This is set to `NO` the first time the Google Analytics SDK is used on a
 device, and is persisted thereafter.
 */
@property(nonatomic, assign) BOOL optOut;

/*!
 If this value is positive, tracking information will be automatically
 dispatched every dispatchInterval seconds. Otherwise, tracking information must
 be sent manually by calling dispatch.

 By default, this is set to `120`, which indicates tracking information should
 be dispatched automatically every 120 seconds.
 */
@property(nonatomic, assign) NSTimeInterval dispatchInterval;

/*!
 When this is 'YES', no tracking information will be sent. Defaults to 'NO'.
 */
@property(nonatomic, assign) BOOL dryRun;

/*! Get the shared instance of the Google Analytics for iOS class. */
+ (GAM *)sharedInstance;

/*!
 Creates or retrieves a GAITracker implementation with the specified name and
 tracking ID. If the tracker for the specified name does not already exist, then
 it will be created and returned; otherwise, the existing tracker will be
 returned. If the existing tracker for the respective name has a different
 tracking ID, that tracking ID is not changed by this method. If defaultTracker
 is not set, it will be set to the tracker instance returned here.

 @param name The name of this tracker. Must not be `nil` or empty.

 @param trackingId The tracking ID to use for this tracker.  It should be of
 the form `UA-xxxxx-y`.

 @return A GAITracker associated with the specified name. The tracker
 can be used to send tracking data to Google Analytics. The first time this
 method is called with a particular name, the tracker for that name will be
 returned, and subsequent calls with the same name will return the same
 instance. It is not necessary to retain the tracker because the tracker will be
 retained internally by the library.

 If an error occurs or the name is not valid, this method will return
 `nil`.
 */
- (id<GAMTracker>)trackerWithName:(NSString *)name trackingId:(NSString *)trackingId;

/*!
 Creates or retrieves a GAITracker implementation with name equal to
 the specified tracking ID. If the tracker for the respective name does not
 already exist, it is created, has it's tracking ID set to |trackingId|,
 and is returned; otherwise, the existing tracker is returned. If the existing
 tracker for the respective name has a different tracking ID, that tracking ID
 is not changed by this method. If defaultTracker is not set, it is set to the
 tracker instance returned here.

 @param trackingId The tracking ID to use for this tracker.  It should be of
 the form `UA-xxxxx-y`. The name of the tracker will be the same as trackingID.

 @return A GAITracker associated with the specified trackingID. The tracker
 can be used to send tracking data to Google Analytics. The first time this
 method is called with a particular trackingID, the tracker for the respective
 name will be returned, and subsequent calls with the same trackingID
 will return the same instance. It is not necessary to retain the tracker
 because the tracker will be retained internally by the library.

 If an error occurs or the trackingId is not valid, this method will return
 `nil`.
 */
- (id<GAMTracker>)trackerWithTrackingId:(NSString *)trackingId;

/*!
 Remove a tracker from the trackers dictionary. If it is the default tracker,
 clears the default tracker as well.

 @param name The name of the tracker.
 */
- (void)removeTrackerByName:(NSString *)name;

/*!
 Dispatches any pending tracking information.

 This calls dispatchAndWaitUntilDone: with NO so it will not block.
 */
- (void)dispatch;

/*!
 Dispatches any pending tracking information.

 It would be wise to call this when application is exiting to initiate the
 submission of any unsubmitted tracking information. Note that this does not
 have any effect on dispatchInterval, and can be used in conjuntion with
 periodic dispatch.

 @param waitUntilDone YES if you want to block the calling thread until the all
 events have been dispatched.
 */
- (void)dispatchAndWaitUntilDone:(BOOL)waitUntilDone;

@end
