//
//  GAMDictionaryBuilder.h
//  GAMKit
//
//  Created by Isaac Wankerl on 11/12/13.
//  Copyright (c) 2013 Evernote Corp. All rights reserved.
//

#import <Foundation/Foundation.h>

/*!
 * Helper class to build a dictionary of hit parameters and values.
 * <br>
 * Examples:
 * <code>
 * id<GAMTracker> t = // get a tracker.
 * [t send:[[[GAMDictionaryBuilder createEventWithCategory:@"EventCategory"
 *                                                  action:@"EventAction"
 *                                                   label:nil
 *                                                   value:nil]
 *     set:@"dimension1" forKey:[GAMFields customDimensionForIndex:1]] build]];
 * </code>
 * This will send an event hit type with the specified parameters
 * and a custom dimension parameter.
 * <br>
 * If you want to send a parameter with all hits, set it on GAMTracker directly.
 * <code>
 * [t set:kGAMScreenName value:@"Home"];
 * [t send:[[GAMDictionaryBuilder createSocialWithNetwork:@"Google+"
 *                                                 action:@"PlusOne"
 *                                                 target:@"SOME_URL"] build]];
 * [t send:[[GAMDictionaryBuilder createSocialWithNetwork:@"Google+"
 *                                                 action:@"Share"
 *                                                 target:@"SOME_POST"] build]];
 * [t send:[[GAMDictionaryBuilder createSocialWithNetwork:@"Google+"
 *                                                 action:@"HangOut"
 *                                                 target:@"SOME_CIRCLE"]
 *     build]];
 * </code>
 * You can override a value set on the tracker by adding it to the dictionary.
 * <code>
 * [t set:kGAMScreenName value:@"Home"];
 * [t send:...];
 * [t send[[[GAMDictionaryBuilder createEventWithCategory:@"click"
 *                                                 action:@"popup"
 *                                                  label:nil
 *                                                  value:nil]
 *     set:@"popup title" forKey:kGAMScreenName] build]];
 * </code>
 * The values set via [GAMDictionaryBuilder set] or
 * [GAMDictionaryBuilder setAll] will override any existing values in the
 * GAMDictionaryBuilder object (i.e. initialized by
 * [GAMDictionaryBuilder createXYZ]). e.g.
 * <code>
 * GAMDictionaryBuilder *m =
 *     GAMDictionaryBuilder createTimingWithCategory:@"category"
 *                                          interval:@0
 *                                              name:@"name"
 *                                             label:nil];
 * [t send:[m.set:@"10" forKey:kGAMTimingVar] build];
 * [t send:[m.set:@"20" forKey:kGAMTimingVar] build];
 * </code>
 */

@interface GAMDictionaryBuilder : NSObject
{
    NSMutableDictionary *_keyValueMap;
}

- (GAMDictionaryBuilder *)set:(NSString *)value forKey:(NSString *)key;

/*!
 * Copies all the name-value pairs from params into this object, ignoring any
 * keys that are not NSString and any values that are neither NSString or
 * NSNull.
 */
- (GAMDictionaryBuilder *)setAll:(NSDictionary *)params;

/*!
 * Returns the value for the input parameter paramName, or nil if paramName
 * is not present.
 */
- (NSString *)get:(NSString *)paramName;

/*!
 * Return an NSMutableDictionary object with all the parameters set in this
 */
- (NSMutableDictionary *)build;

/*!
 Returns a GAMDictionaryBuilder object with parameters specific to an appview
 hit.

 Note that using this method will not set the screen name for followon hits.  To
 do that you need to call set:kGAMDescription value:<screenName> on the
 GAMTracker instance.
 */
+ (GAMDictionaryBuilder *)createAppView;

/*!
 Returns a GAMDictionaryBuilder object with parameters specific to an event hit.
 */
+ (GAMDictionaryBuilder *)createEventWithCategory:(NSString *)category action:(NSString *)action label:(NSString *)label value:(NSNumber *)value;

/*!
 Returns a GAMDictionaryBuilder object with parameters specific to a page view hit.
 */
+ (GAMDictionaryBuilder *)createPageViewWithPage:(NSString *)pageName;

@end
