//
//  CoreSkitchDocument.h
//  CoreSkitch
//
//  Created by Josef W. Wankerl on 7/1/13.
//  Copyright 2014 Evernote Corporation. All rights reserved.
//

#import <Foundation/Foundation.h>

@class SkitchExportFormat;

/**
	\brief Creates, loads, and saves Skitch documents.
	\details This is the main interface into creating, loading, and saving Skitch documents.
*/

@interface CoreSkitchDocument : NSObject

/**
	\brief The title of the document.
*/
@property(nonatomic, copy) NSString *title;

/**
	\brief The number of pages in the document.
	\details For image documents, the pageCount is one.  For PDF documents, the pageCount is the number of PDF pages.
*/
@property(nonatomic, readonly) NSUInteger pageCount;

/**
	\brief Creates a CoreSkitchDocument instance for the given Skitch document file URL.
	\details This loads a Skitch document and prepares it for use with a CoreSkitchView.
		Most common bitmap based formats are supported as well as PNG and JPG files with embedded Skitch document information and PDF files.
	\param url The file URL specifying a document Skitch knows how to load.
	\param usedFormat Upon return, if the URL is read successfully, contains a pointer to an object describing the format of the URL.
		If you are not interested in the URL format, pass in NULL.
	\param error If an error occurs, upon returns contains an NSError object that describes the problem.
		If you are not interested in possible errors, pass in NULL.
	\return An instance of CoreSkitchDocument.
*/
- (instancetype)initWithURL:(NSURL *)url usedFormat:(SkitchExportFormat * __autoreleasing *)usedFormat error:(NSError * __autoreleasing *)error;

/**
	\brief Creates a CoreSkitchDocument instance with the given size and scale.
	\details This creates a Skitch document without any background image and prepares it for use with a CoreSkitchView.
	\param size The initial size of the Skitch document, in pixels.
	\param scale The initial device scale for the Skitch document.  Use 1.0 for normal images, 2.0 for retina images.
	\return An instance of CoreSkitchDocument.
*/
- (instancetype)initWithSize:(CGSize)size scale:(CGFloat)scale;

/**
	\brief Creates a CoreSkitchDocument instance with the given image.
	\details This creates a Skitch document with the given background image and prepares it for use with a CoreSkitchView.
		If storeAsLossy is YES, the image is allowed to be compressed in such a fashion as it may not match its original
		image upon uncompression.  It is generally useful to treat JPG background images as lossy.  If storeAsLossy is NO,
		the image is kept intact, pixel-for-pixel.  It is generally useful to treat PNG screenshots as lossless.
	\param image The initial image to use for background of the Skitch document.
	\param scale The initial device scale for the Skitch document.  Use 1.0 for normal images, 2.0 for retina images.
	\param storeAsLossy YES to treat the image internally as a lossy compressed image, NO to treat the image as lossless.
	\return An instance of CoreSkitchDocument.
*/
- (instancetype)initWithBackgroundImage:(CGImageRef)image scale:(CGFloat)scale storeAsLossy:(BOOL)storeAsLossy;

/**
	\brief Exports a marked up document to a given file URL.
	\details This saves a Skitch document to the given file URL with the specified options.
		Most common bitmap based formats are supported as well as PNG and JPG files with embedded Skitch document information and PDF files.
		The default options are to compress JPG files at 80% and to not show an author name if a summary page is requested.
	\param url The file URL specifying the target for the save operation.
	\param outputFormat The output format to use when writing the document.
	\param options The extra options to use when writing the document.
		If you want the default options, pass in nil.
	\param error On return, if the document content could not be written, a pointer to an error object that encapsulates the reason it could not be written.
		If you are not interested in possible errors, pass in NULL.
	\return YES if the document content could be written; otherwise, NO.
*/
- (BOOL)writeToURL:(NSURL *)url outputFormat:(SkitchExportFormat *)outputFormat options:(NSDictionary *)options error:(NSError * __autoreleasing *)error;

/**
	\brief Exports a flattened document to a given file URL after stripping out all annotations.
	\details This saves a Skitch document to the given file URL after removing all annotations.
		The resulting document is completely flat with no DOM embedded.
	\param url The file URL specifying the target for the save operation.
	\param error On return, if the document content could not be written, a pointer to an error object that encapsulates the reason it could not be written.
		If you are not interested in possible errors, pass in NULL.
	\return YES if the document content could be written; otherwise, NO.
*/
- (BOOL)writeCleanUnannotatedToURL:(NSURL *)url error:(NSError * __autoreleasing *)error;

/**
	\brief Makes a flattened image from the document and returns its data.
	\details This creates an image from the document.
		Most common bitmap based formats, but not PNG and JPG files with embedded Skitch document information.
		PDF output is not supported either.
		The default options are to compress JPG files at 80% and to create an image from the very first page of a PDF.
	\param outputFormat The output format to use when creating the image.
	\param options The extra options to use when creating the image.
		If you want the default options, pass in nil.
	\return The data representing a flattened image of the document.
*/
- (NSData *)imageRepresentationWithFormat:(SkitchExportFormat *)outputFormat options:(NSDictionary *)options;

@end

/**
	\addtogroup Constants
	@{
*/

/**
	\var CoreSkitchDocumentOptionCompression
	\brief Option specifying the quality of compression to use with a compressed bitmap image format.
	\details A NSNumber object with a value between 0.0 and 1.0.  If absent, a default of 0.8 is used.
*/

extern NSString * const CoreSkitchDocumentOptionCompression;

/**
	\var CoreSkitchDocumentOptionPDFAuthor
	\brief Option specifying the name of the author to use on a PDF summary page.
	\details A NSString object with the name of the author.  If absent or an empty string, no author name will be used.
*/

extern NSString * const CoreSkitchDocumentOptionPDFAuthor;

/**
	\var CoreSkitchDocumentOptionPDFSummaryTitle
	\brief Option specifying the summary description title for the PDF summary page.
	\details A NSString object with the title for the summary page.  If absent or an empty string, the summary title "PDF Summary" will be used.
*/

extern NSString * const CoreSkitchDocumentOptionPDFSummaryTitle;

/**
	\var CoreSkitchDocumentOptionPDFPageIndex
	\brief Option specifying the index of the PDF page used to create an image.
	\details A NSNumber object with the index of the PDF page.  If absent, a default of 0 is used which corresponds to the very first page.
		This option is only valid with the imageRepresentationWithFormat: method and is ignored with all other output methods.
*/

extern NSString * const CoreSkitchDocumentOptionPDFPageIndex;

/** @} */
