//
//  CoreSkitchView.h
//  CoreSkitch
//
//  Created by Josef W. Wankerl on 7/1/13.
//  Copyright 2014 Evernote Corporation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <CoreSkitch/CoreSkitchDocument.h>
#import <CoreSkitch/SkitchExportFormat.h>

/**
	\brief Specifies the zoom mode of the view. This setting controls how the view's scale factor changes when then view size changes.
	\see scaleFactor
	\see aspectFitScaleFactor
	\see widthFitScaleFactor
	\see zoomMode
*/

typedef enum CoreSkitchViewZoomMode : NSUInteger
{
	/**
		\brief No zoom mode is set. As the view is resized, the view scale factor is not modified.
	*/
	CoreSkitchViewZoomModeRegular = 0,

	/**
		\brief As the view is resized, the view scale factor changes so the composition fills with aspect fit.
	*/
	CoreSkitchViewZoomModeAspectFit = 1,

	/**
		\brief As the view is resized, the view scale factor changes so the composition fills with the whole width.
	*/
	CoreSkitchViewZoomModeFitPageWidth = 2

} CoreSkitchViewZoomMode;


/**
	\brief View that displays an editable Skitch document.
	\details This is the main view used for interacting with Skitch documents.
*/

@interface CoreSkitchView : NSView

/**
	\brief The Skitch document to display.
	\details This is the Skitch document the view displays and edits.
*/
@property(nonatomic, strong) CoreSkitchDocument *document;

/**
	\brief The current scale factor for the view.
	\details The default value is 1.0, corresponding to actual size.
		Explicitly setting the scaleFactor causes the zoomMode to become CoreSkitchViewZoomModeRegular.
	\see aspectFitScaleFactor
	\see widthFitScaleFactor
	\see zoomMode
*/
@property(nonatomic, assign) CGFloat scaleFactor;

/**
	\brief The current zoom mode for the view.
	\see CoreSkitchViewZoomMode
	\see scaleFactor
	\see aspectFitScaleFactor
	\see widthFitScaleFactor
*/
@property(nonatomic, assign) CoreSkitchViewZoomMode zoomMode;

/**
	\brief The format to use for putting images on the pasteboard.
	\details Only flat images are placed on the pasteboard, any DOM embedding is ignored.
*/
@property(nonatomic, copy) SkitchExportFormat *imagePasteboardCopyFormat;

/**
	\brief Is the PDF canvas ignoring changes to the underlying document model.
	\details Due to the way that documents are saved, the underlying document model may be
		updated during the save process.  The PDF canvas is watching for changes to make
		sure the page with a change is scrolled into view and also causes the PDF thumbnails
		area to be regenerated.  During a save of the document, set ignorePDFSaveChanges to YES
		so unwanted PDF canvas scrolling does not occur and unwanted thumbnail regeneration
		does not occur.  Then once saving is finished and regular canvas editing is once
		again ready, set ignorePDFSaveChanges to NO.  It does not hurt to change this setting
		on a bitmap canvas; in that case it is simply ignored.
*/
@property(nonatomic, assign) BOOL ignorePDFSaveChanges;

/**
	\brief Returns the undo manager for the current tool.
	\details For most tools, the undo manager is the same as would be returned by calling undoManager
		on CoreSkitchView.  During a crop operation, the a separate undo manager is used to ensure that
		operations for changing the crop area are kept on a separate undo stack.  The window
		delegate method windowWillReturnUndoManager: should generally return the undoManagerForCurrentTool
		to ensure the proper manager is seen.
	\see windowWillReturnUndoManager:
*/
@property(nonatomic, readonly) NSUndoManager *undoManagerForCurrentTool;

/**
	\brief Calculates and returns a CGFloat value indicating the scale factor for the view that
		would fill the whole view using an aspect fit.
	\return A CGFloat value of the calculated scale factor.
	\see scaleFactor
*/
- (CGFloat)aspectFitScaleFactor;

/**
	\brief Calculates and returns a CGFloat value indicating the scale factor for the view that
		would fill the whole width of the view.
	\return A CGFloat value of the calculated scale factor.
	\see scaleFactor
*/
- (CGFloat)widthFitScaleFactor;

/**
	\brief Returns a Boolean value indicating whether the user can change the scale factor-that is zoom in or out.
	\details This is NO when the canvas is using the crop or resize tool, otherwise YES.
	\return YES if zooming is allowed, otherwise NO.
*/
- (BOOL)canZoom;

/**
	\brief Returns a Boolean value indicating whether the user can magnify the view—that is, zoom in.
	\return YES if zoom in is allowed, otherwise NO.
*/
- (BOOL)canZoomIn;

/**
	\brief Zooms in by increasing the scaling factor.
	\details Each invocation of zoomIn muliplies the scaling factor by the square root of 2.
	\param sender The object calling this method.
	\see scaleFactor
*/
- (IBAction)zoomIn:(id)sender;

/**
	\brief Returns a Boolean value indicating whether the user can view an expanded area—that is, zoom out.
	\return YES if zoom out is allowed, otherwise NO.
*/
- (BOOL)canZoomOut;

/**
	\brief Zooms in by increasing the scaling factor.
	\details Each invocation of zoomOut divides the scaling factor by the square root of 2.
	\param sender The object calling this method.
	\see scaleFactor
*/
- (IBAction)zoomOut:(id)sender;

/**
	\brief Remove the selection from the user interface and write it to the pasteboard.
	\param sender The object calling this method.
*/
- (IBAction)cut:(id)sender;

/**
	\brief Copy the selection to the pasteboard.
	\param sender The object calling this method.
*/
- (IBAction)copy:(id)sender;

/**
	\brief Copy the entire image to the pasteboard, even if there are selected items.
	\param sender The object calling this method.
*/
- (IBAction)copyImage:(id)sender;

/**
	\brief Read data from the pasteboard and display it in the user interface.
	\param sender The object calling this method.
*/
- (IBAction)paste:(id)sender;

/**
	\brief Remove the selection from the user interface.
	\param sender The object calling this method.
*/
- (IBAction)delete:(id)sender;

/**
	\brief Select all objects in the current view.
	\param sender The object calling this method.
*/
- (IBAction)selectAll:(id)sender;

/**
	\brief Deselect all objects in the current view.
*/
- (void)clearSelection;

/**
	\brief Removes all non-destructive annotations from all pages of the current document.
	\param sender The object calling this method.
*/
- (IBAction)removeAllAnnotations:(id)sender;

/**
	\brief Flips the bitmap canvas image along the vertical centerline (creates a mirror image).
	\details This method only works on a bitmap canvas.  For a PDF canvas, this method does nothing.
	\param sender The object calling this method.
*/
- (IBAction)flipImageHorizontal:(id)sender;

/**
	\brief Rotates the bitmap canvas image 90 degrees clockwise.
	\details This method only works on a bitmap canvas.  For a PDF canvas, this method does nothing.
	\param sender The object calling this method.
*/
- (IBAction)rotateImageClockwise:(id)sender;

/**
	\brief Rotates the bitmap canvas image 90 degrees counterclockwise.
	\details This method only works on a bitmap canvas.  For a PDF canvas, this method does nothing.
	\param sender The object calling this method.
*/
- (IBAction)rotateImageCounterclockwise:(id)sender;

/**
	\brief Rotates the current page 90 degrees clockwise.
	\details For a bitmap canvas, the entire image is rotated.  For a PDF canvas, the current page is rotated.
	\param sender The object calling this method.
*/
- (IBAction)rotatePageClockwise:(id)sender;

/**
	\brief Rotates the current page 90 degrees counterclockwise.
	\details For a bitmap canvas, the entire image is rotated.  For a PDF canvas, the current page is rotated.
	\param sender The object calling this method.
*/
- (IBAction)rotatePageCounterclockwise:(id)sender;

/**
	\brief Deletes the current PDF page.
	\details This method only works on a PDF canvas.  For a bitmap canvas, this method does nothing.
	\param sender The object calling this method.
*/
- (IBAction)deletePage:(id)sender;

/**
	\brief Scrolls to the beginning of the document, without changing the selection.
*/
- (void)scrollToBeginningOfDocument:(id)sender;

/**
	\brief Scrolls to the end of the document, without changing the selection.
*/
- (void)scrollToEndOfDocument:(id)sender;

/**
	\brief Causes the view (or one of its embedded text fields) to resign the first responder status.
	\details This method looks at the current view and its subview hierarchy for the text field that is currently the first responder.
		If it finds one, it asks that text field to resign as first responder.
*/
- (void)endEditing;

/**
	\brief Generates a print operation suitable for printing the view content.
	\param printInfo The NSPrintInfo object to use for the print operation.
	\param error On return, if the print operation could not be created, a pointer to an error object that encapsulates the reason it could not be created.
	\return The print operation, or nil if unsuccessful.
*/
- (NSPrintOperation *)printOperationForPrintInfo:(NSPrintInfo *)printInfo error:(NSError * __autoreleasing *)error;

@end


/**
	\addtogroup Constants
	@{
*/

/**
	\var CoreSkitchViewArrowheadAtBeginningOfShapeKey
	\brief Key for setValue:forKey and valueForKey: to get and configure the position of the arrowhead during drawing.
		The default value is NO.
	\details The value is a NSNumber object representing a BOOL value.
*/

extern NSString * const CoreSkitchViewArrowheadAtBeginningOfShapeKey;

/**
	\var CoreSkitchViewExpandableCanvasKey
	\brief Key for setValue:forKey and valueForKey: to get and configure the expandable canvas.  The default value is NO.
	\details The value is a NSNumber object representing a BOOL value.  Expandable canvas is only valid for a bitmap canvas, not PDF.
*/

extern NSString * const CoreSkitchViewExpandableCanvasKey;

/**
	\var CoreSkitchViewPDFScaleOnePointEqualsOneScreenPixelKey
	\brief Key for setValue:forKey and valueForKey: to get and configure the how PDF 100% scale is defined.
		The default value is NO, which means that PDF 100% scale is defined as "Size on screen equals size on printout".
		A value of YES is defined as "1 point equals 1 screen pixel".
	\details The value is a NSNumber object representing a BOOL value.
*/

extern NSString * const CoreSkitchViewPDFScaleOnePointEqualsOneScreenPixelKey;

/**
	\var CoreSkitchViewTypingCreatesTextAnnotationKey
	\brief Key for setValue:forKey and valueForKey: to get and configure whether typing automatically creates a text annotation or not.
	\details The value is a NSNumber object representing a BOOL value.  The default value is NO.
		YES to have typing automatically create a text annotation; NO to have typing ignored.
		Automatically creating text annotations is only valid for a bitmap canvas, not PDF.
*/

extern NSString * const CoreSkitchViewTypingCreatesTextAnnotationKey;

/**
	\var CoreSkitchViewTypingCreatesTextAnnotationIgnoredKeycodesKey
	\brief Key for setValue:forKey and valueForKey: to get and configure which keys do not trigger creation of an automatic text annotation.
	\details The value is a NSSet object where each element is a NSNumber object that represents a keycode.
		When a keyDown: event occurs, the keycode is checked to see if it is in the set, and if so then no automatic text annotation is created.
		The default value is a set containing keycodes for the delete key, forward delete key, and the return key.
*/

extern NSString * const CoreSkitchViewTypingCreatesTextAnnotationIgnoredKeycodesKey;

/**
	\var CoreSkitchViewStandardNewlineTextInputKey
	\brief Key for setValue:forKey and valueForKey: to get and configure the return key behavior during text input.
	\details The value is a NSNumber object representing a BOOL value.  The default value is NO.
		YES to have newlines area added with Option-Return (standard Mac behavior);
		NO, to have newlines added with Return (legacy Skitch behavior).
*/

extern NSString * const CoreSkitchViewStandardNewlineTextInputKey;

/**
	\var CoreSkitchViewToolBarDividerKey
	\brief Key for setValue:forKey and valueForKey: to get and configure whether or not the tool bar has a
		divider line on its righthand side.  The default value is NO.
	\details The value is a NSNumber object representing a BOOL value.
*/

extern NSString * const CoreSkitchViewToolBarDividerKey;

/**
	\var CoreSkitchViewToolBarShowEmoticonStampsKey
	\brief Key for setValue:forKey and valueForKey: to get and configure whether or not the tool bar has the
		emoticon stamps.  The default value is NO.  If emoticon stamps are desired, this value must be set
		to YES after every time a new document has been set.
	\details The value is a NSNumber object representing a BOOL value.
*/

extern NSString * const CoreSkitchViewToolBarShowEmoticonStampsKey;

/**
	\var CoreSkitchViewPDFShowsThumbnailPopUpKey
	\brief Key for setValue:forKey and valueForKey: to get and configure whether or not the PDF thumbnail area
		shows a pop-up menu to select all pages vs. annotated pages.  The default value is YES.
	\details The value is a NSNumber object representing a BOOL value.
	\see CoreSkitchViewPDFThumbnailPopUpButtonKey
*/

extern NSString * const CoreSkitchViewPDFShowsThumbnailPopUpKey;

/**
	\var CoreSkitchViewPDFThumbnailPopUpButtonKey
	\brief Key for valueForKey: to get a NSPopUpButton control to use outside of the PDF thumbnail area to
		select all pages vs. annotated pages.  Using setValue:forKey does nothing.  It is generally only
		useful to get the control when CoreSkitchViewPDFShowsThumbnailPopUpKey has been set to NO.
	\details The value is a NSPopUpButton object.
	\see CoreSkitchViewPDFShowsThumbnailPopUpKey
*/

extern NSString * const CoreSkitchViewPDFThumbnailPopUpButtonKey;

/**
	\var CoreSkitchViewCanvasStateKey
	\brief Key for setValue:forKey and valueForKey: to get and configure the tool bar and related properties.
	\details The value is an opaque NSData object.  The canvas state for bitmaps is different than that for
		a PDF canvas.  The two should be saved/restored separately.  Trying to set a bitmap state on a PDF
		canvas, or vice versa, is simply ignored.  Restoring a bitmap state where the crop, resize, or
		pixelate tool has been selected causes the previous regular tool to be selected.
*/

extern NSString * const CoreSkitchViewCanvasStateKey;

/**
	\var CoreSkitchViewInterestingEventCategoryKey
	\brief Key for the CoreSkitchViewInterestingEventDidOccur notification userInfo dictionary.
	\details The value is a NSString object representing an interesting event category.
		The category value is in the format suitable to report as a Google Analytics data point.
	\see CoreSkitchViewInterestingEventDidOccur
*/

extern NSString * const CoreSkitchViewInterestingEventCategoryKey;

/**
	\var CoreSkitchViewInterestingEventActionKey
	\brief Key for the CoreSkitchViewInterestingEventDidOccur notification userInfo dictionary.
	\details The value is a NSString object representing an interesting event action.
		The action value is in the format suitable to report as a Google Analytics data point.
	\see CoreSkitchViewInterestingEventDidOccur
*/

extern NSString * const CoreSkitchViewInterestingEventActionKey;

/**
	\var CoreSkitchViewInterestingEventLabelKey
	\brief Key for the CoreSkitchViewInterestingEventDidOccur notification userInfo dictionary.
	\details The value is a NSString object representing an interesting event label.
		The label value is in the format suitable to report as a Google Analytics data point.
	\see CoreSkitchViewInterestingEventDidOccur
*/

extern NSString * const CoreSkitchViewInterestingEventLabelKey;

/**
	\var CoreSkitchViewInterestingEventValueKey
	\brief Key for the CoreSkitchViewInterestingEventDidOccur notification userInfo dictionary.
	\details The value is a NSNumber object representing an interesting event value.
		The value is in the format suitable to report as a Google Analytics data point.
	\see CoreSkitchViewInterestingEventDidOccur
*/

extern NSString * const CoreSkitchViewInterestingEventValueKey;

/** @} */


/**
	\addtogroup Notifications
	@{
*/

/**
	\var CoreSkitchViewScaleFactorDidChange
	\brief Notification sent when scaleFactor has changed.
	\details The object is a reference to the CoreSkitchView. There is no userInfo dictionary.
*/

extern NSString * const CoreSkitchViewScaleFactorDidChange;

/**
	\var CoreSkitchViewInterestingEventDidOccur
	\brief Notification sent when an interesting internal event has occurred.
	\details The object is a reference to the CoreSkitchView.
		The userInfo dictionary may contain four optional entries to describe the interesting event.
	\see CoreSkitchViewInterestingEventCategoryKey
	\see CoreSkitchViewInterestingEventActionKey
	\see CoreSkitchViewInterestingEventLabelKey
	\see CoreSkitchViewInterestingEventValueKey
*/

extern NSString * const CoreSkitchViewInterestingEventDidOccur;

/** @} */
